// Modules to control application life and create native browser window
const { initBrowserEnv } = require("./src/browser_env/index");
const { app, BrowserWindow } = require('electron');
const remoteCall = require("./src/remoteCall");
const Rpc = require("./src/electron/rpc");
const fetch = require('node-fetch');
const fs = require("fs");
const path = require("path");
const config = require("./config.json");
const { loadVmJS, mkDir } = require("./src/tools");


let userDataPath = app.getPath("userData");
// let userDataPath = app.getAppPath();
initBrowserEnv(userDataPath);

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;

function createWindow() {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 1800,
    height: 900,
    webPreferences: {
      preload: path.join(__dirname, 'preload.js')
    },
    frame: false,
    show: false
  });
  Rpc.setSender(mainWindow.webContents);
  mainWindow.loadURL(getInitUrl(config.init_url, config.game_name));
  mainWindow.once('ready-to-show', () => {
    // 这里延迟加载是因为每次第一次启动  检查更新需要耗时2s左右  导致会有2s的白屏
    setTimeout(() => {
      mainWindow.show()
    }, 1500)
  });
  loadVmJS(userDataPath).then(() => {
    console.log("vm 初始化完成");
  });
  mainWindow.on('closed', function () {
    mainWindow = null
  })

  mainWindow.webContents.on('new-window', (event, url, frameName, disposition, options, additionalFeatures) => {
    // open window as modal
    event.preventDefault();
    Object.assign(options, {
      frame: true
    });
    const win = new BrowserWindow(options);
    win.once('ready-to-show', () => win.show());
    if (!options.webContents) {
      win.loadURL(url); // existing webContents will be navigated automatically
    }
    event.newGuest = win;
  })
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow)

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On macOS it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') app.quit()
})

app.on('activate', function () {
  // On macOS it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) createWindow()
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.


const getInitUrl = (init_url, game_name) => {
  if (init_url.startsWith("http") || init_url.startsWith("https")) return init_url;
  let urlPath = path.resolve(userDataPath, "static", init_url);
  if (!fs.existsSync(urlPath)) { // 不存在 去原始目录寻找
    urlPath = path.resolve(__dirname, "assets", init_url);
  }
  let url = require('url').format({
    protocol: 'file',
    slashes: true,
    pathname: urlPath
  });
  if (game_name) url = `${url}?game=${game_name}`;
  return url;
}

const restartApp = (restartPath) => {
  const arr = restartPath.split("?");
  let search = arr[1];
  search = search ? `?${search}` : '';
  console.log("restartApp url = ", `${getInitUrl(arr[0])}${search}`);
  mainWindow.loadURL(`${getInitUrl(arr[0])}${search}`)
}

/**
 * 获取本地包游戏url
 * 这里始终从服务器获取最新的index.html
 * 然后写入本地
 * 再通过file协议加载
 */
const getLocalGameConfig = async () => {
  try {
    const res = await fetch(config.game_url);
    if (!res.ok) throw new Error(res.statusText);
    const content = await res.text();
    console.log("获取游戏远端index.html成功！！！！！");
    const gamePath = path.resolve(userDataPath, "static", config.game_local_url);
    mkDir(gamePath);
    fs.writeFileSync(gamePath, content, "utf-8");
    let url = require('url').format({
      protocol: 'file',
      slashes: true,
      pathname: gamePath
    });
    return { url, title: config.game_title };
  } catch (err) {
    let notFindUrl = require('url').format({
      protocol: 'file',
      slashes: true,
      pathname: path.resolve(__dirname, "html_404", "game_not_find.html")
    });
    return { url: notFindUrl, title: config.game_title }
  }
}




const executeJs = (args) => {
  args = args.map(arg => {
    if (typeof arg === "string") {
      return `'${arg}'`;
    } else {
      return arg;
    }
  })
  // console.log("executeJs =======", `window.handle_jsintercept_callback(${args.join(",")})`);
  mainWindow.webContents.executeJavaScript(`window.handle_jsintercept_callback(${args.join(",")})`);
}

const maximize = () => {
  return mainWindow.maximize();
}

const isMaximized = () => {
  return mainWindow.isMaximized();
}

const restore = () => {
  return mainWindow.restore();
}

const close = () => {
  return mainWindow.close();
}

const reload = () => {
  return mainWindow.reload();
}

const minimize = () => {
  return mainWindow.minimize();
}

exports.restartApp = restartApp;
exports.getLocalGameConfig = getLocalGameConfig;
exports.appPath = app.getAppPath();
exports.userDataPath = userDataPath;
exports.executeJs = executeJs;
exports.maximize = maximize;
exports.isMaximized = isMaximized;
exports.restore = restore;
exports.close = close;
exports.reload = reload;
exports.minimize = minimize;


